@@ -4,8 +4,8 @@ use Perl::Repository::APC;
 use strict;
 use warnings;
 
-my $Id = q$Id: BAP.pm 220 2006-10-22 10:51:44Z k $;
-our $VERSION = sprintf "%.3f", 1 + substr(q$Rev: 220 $,4)/1000;
+my $Id = q$Id: BAP.pm 279 2008-01-20 12:41:29Z k $;
+our $VERSION = sprintf "%.3f", 1 + substr(q$Rev: 279 $,4)/1000;
 
 sub new {
   unless (@_ == 2){
@@ -24,10 +24,10 @@ sub new {
 }
 
 sub translate {
-  my($self,$branch,$ver,$lev) = @_;
+  my($self,$branch,$baseperl,$patchlevel) = @_;
   die sprintf "%s -> translate called without a branch argument", __PACKAGE__
       unless $branch;
-  my($next, $this, $prev, @patches, @ver);
+  my($prev, $nextperl, @patches, @ver);
   my $apc = $self->{APC};
   if ($branch eq "perl") {
     $prev = "0";
@@ -48,55 +48,55 @@ sub translate {
   }
   @ver = $prev;
   for (
-       $next = $apc->first_in_branch($branch);
+       my $next = $apc->first_in_branch($branch);
        $next;
        $next = $apc->next_in_branch($next)
       ) {
-    $this = $next;
+    $nextperl = $next;
     @patches = @{$apc->patches($next)};
     push @ver, $next;
-    if ($lev && $lev >= $patches[0] && $lev <= $patches[-1]){
-      if (defined $ver && length $ver &&
-          grep { $_ eq $ver } @ver) {
-        unless ($prev eq $ver){
-          die "Fatal error: patch $lev is outside the patchset for $ver\n";
+    if ($patchlevel && $patchlevel >= $patches[0] && $patchlevel <= $patches[-1]){
+      if (defined $baseperl && length $baseperl &&
+          grep { $_ eq $baseperl } @ver) {
+        unless ($prev eq $baseperl){
+          die "Fatal error: patch $patchlevel is outside the patchset based on $baseperl\n";
         }
       }
       last;
-    } elsif (defined $ver && length($ver)) {
-      if ($ver eq "0") {
+    } elsif (defined $baseperl && length($baseperl)) {
+      if ($baseperl eq "0") {
         if ($ver[0] eq "0") {
           last;
         } else {
           die "Fatal error: 0 is not starting point for branch $branch\n";
         }
       } else {
-        last if $prev && $ver eq $prev || @ver>1 && $ver eq $ver[-2];
+        last if $prev && $baseperl eq $prev || @ver>1 && $baseperl eq $ver[-2];
       }
     }
     $prev = $next;
   }
-  if (defined $ver && length $ver) {
-    if ($ver eq "0") {
+  if (defined $baseperl && length $baseperl) {
+    if ($baseperl eq "0") {
       # always OK?
     } else {
-      unless (grep { $_ eq $ver } @ver){
-        die "Fatal error: $ver is not part of branch $branch";
+      unless (grep { $_ eq $baseperl } @ver){
+        die "Fatal error: $baseperl is not part of branch $branch";
       }
     }
   } else {
     if (@ver > 1) {
-      $ver = $ver[-2];
+      $baseperl = $ver[-2];
     } elsif (@ver == 1) {
-      $ver = $ver[0];
-      $ver =~ s/1$/0/;
+      $baseperl = $ver[0];
+      $baseperl =~ s/1$/0/;
     } else {
       die "Could not determine base perl version";
     }
   }
-  if ($lev) {
-    unless (grep { $_ eq $lev } @patches){
-      my @neighbors = $self->neighbors($lev,\@patches);
+  if ($patchlevel) {
+    unless (grep { $_ eq $patchlevel } @patches){
+      my @neighbors = $self->neighbors($patchlevel,\@patches);
       my $tellmore;
       if (@neighbors) {
         if (@neighbors == 1) {
@@ -107,14 +107,15 @@ sub translate {
       } else {
         $tellmore = "Range is from $patches[0] to $patches[-1]";
       }
-      die "Fatal error: patch $lev is not part of the patchset for $ver
+      die "Fatal error: patch $patchlevel is not part of the patchset for $baseperl
     ($tellmore)\n";
     }
   } else {
-    $lev = $patches[-1];
+    $patchlevel = $patches[-1];
   }
-  my $first = $patches[0];
-  return ($ver,$this,$first,$lev);
+  my $firstpatch = $patches[0];
+  my $dir = $apc->get_diff_dir($branch,$patchlevel);
+  return ($baseperl, $nextperl, $firstpatch, $patchlevel, $dir);
 }
 
 sub neighbors {
@@ -149,7 +150,7 @@ Perl::Repository::APC::BAP - Transform the argument to buildaperl
   use Perl::Repository::APC::BAP;
   my $apc = Perl::Repository::APC->new("/path/to/APC");
   my $bap = Perl::Repository::APC::BAP->new($apc);
-  my($version,$prev_version,$patchlevel) = $bap->transform("perl");
+  my($baseperl,$nextperl,$firstpatch,$lastpatch,$dir) = $bap->translate("perl",...);
 
 =head1 DESCRIPTION
 
@@ -158,30 +159,37 @@ object. The resulting object has the following methods:
 
 =over
 
-=item * translate($branch,$version,$patch)
+=item * translate($branch,undef,$patchlevel)
+
+=item * translate($branch,$baseperl)
+
+=item * translate($branch,$baseperl,$patchlevel)
 
 $branch is one of C<perl>, C<maint-5.004>, C<maint-5.005>,
-C<maint-5.6>, C<maint-5.8>, or any other branch that is part of the
-APC. $version is the perl version we want as a base. $patch is a patch
-number that B<must> also be available in the local copy of APC.
+C<maint-5.6>, C<maint-5.8>. $baseperl is the perl version we want as a
+base. $patchlevel is a patch number that B<must> also be available in
+the local copy of APC.
 
-$branch is a mandatory argument, $version and $patch can be omitted.
-If $version is omitted and $patch is given, translate() finds the
-proper version. If patch is omitted and $version is given, translate()
-finds the most recent patch for that base. If both are omitted,
-translate() finds the newest values available for both version and
-patch for that branch. If both are given, translate() checks if the
-values are legal and dies if they aren't.
+$branch is a mandatory argument, $baseperl and $patchlevel can be
+omitted. If $baseperl is omitted and $patchlevel is given, translate()
+finds the proper version. If patch is omitted and $baseperl is given,
+translate() finds the most recent patch for that base. If both are
+omitted, translate() finds the newest values available for both
+version and patch for that branch. If both are given, translate()
+checks if the values are legal and dies if they aren't.
 
-Four values are returned: the perl version that this patch should be
-leading to, the perl version we can use as a base, and the first and
-the last patch number we want. Please see bap.t for examples.
+Five values are returned: the perl version we can use as a base, the
+"next" perl version that this patchset is running to, the first and
+the last patch number we want, and finally the directory where we find
+the patches. The directory was the same as the target perl version up
+to 5.8.0 but got fuzzy from 5.8.1 and later. Please see bap.t for
+examples.
 
 =back
 
 =head1 AUTHOR
 
-andreas.koenig@anima.de
+Andreas Koenig C<< <ANDK> >>
 
 =head1 LICENSE
 
